
#import "ICQueueBase.h"


@implementation ICSingleQueue
@synthesize array,counter;


- (id)init
{
    self = [super init];
    if (self) {
        array = [[NSMutableArray alloc] init];
		counter=0;
    }
    
    return self;
}
- (void)dealloc {
	
	[array release];
	[super dealloc];
}

- (id)pop:(BOOL)popAndRemove :(NSError**)error{
	if (popAndRemove) {
		if ([array count]>0) {
			id element=[array objectAtIndex:0];
			[array removeObjectAtIndex:0];
			return element;
		}else {
			return nil;
		}

	}else {
		if (counter<=[array count]-1){
			id element;
			element=[array objectAtIndex:counter];
			counter++;
			return element;
		}else {
			NSMutableDictionary *errorDetail = [NSMutableDictionary dictionary];
			[errorDetail setValue:@"No element in the Queue to return!" forKey:NSLocalizedDescriptionKey];
			*error = [NSError errorWithDomain:@"myDomain" code:1 userInfo:errorDetail];
			return nil;
		}

	}
}

- (void)push:(id)element {
	
	if (element!= nil) {
    [array addObject:element];
	}
}
- (void)pushElements:(NSArray*)elements {
	
	[array addObjectsFromArray:elements];
}
- (NSMutableArray*)getQueue{

	return [self array];
}

- (void) reset{

	self.counter=0;
}

- (NSInteger)size {
    return [array count];
}

- (BOOL)isEmpty {
    return [array count] == 0;
}

- (void) clean {
    [array removeAllObjects];
}

@end


//----------------------------------------------------


@implementation ICMultipleQueue
@synthesize keys,keysQueue,queue;

- (id)init
{
    self = [super init];
    if (self) {
		keys = [NSMutableArray array];
        keysQueue = [NSMutableDictionary dictionary];
		queue = [NSMutableArray array];
    }
    
    return self;
}
- (void)dealloc {
	
	[queue release];
	[keysQueue release];
	[keys release];
	
	[super dealloc];
}
//----------------------------------------------------
// Makes a new key and a empty Queue.
-(void) addKey: (NSString *) key {
	
	queue = [NSMutableArray array];
	[keys addObject:key];
	[keysQueue setValue:queue forKey:key];
}

// remove a key and its Queue.
-(void) delKey: (NSString *) key {
	
	[keys removeObject:key];
	[keysQueue removeObjectForKey:key];
}

//----------------------------------------------------
// Add object to the certain key.
-(void) push:(NSString *)key: (id)element {
	
	if([[keysQueue allKeys] containsObject:key]){
		
		NSMutableArray *tempQueue=[keysQueue objectForKey:key];
		[tempQueue addObject:element];
		[keysQueue setObject:tempQueue forKey:key];
	}else {
		[self addKey:key];
		NSMutableArray *tempQueue=[keysQueue objectForKey:key];
		[tempQueue addObject:element];
		[keysQueue setObject:tempQueue forKey:key];
	}

}

-(void) pushElements:(NSString *)key: (NSArray *)elements {
	
	for(id element in elements){
		[self push:key :element];
	}
}
//----------------------------------------------------
-(id) pop{
	
	if ([keys count]==0) {
		NSLog(@"No element in the Queue to return!");
		return nil;
	}
	NSString *key=[keys objectAtIndex:0];
	NSMutableArray *tempQueue = [keysQueue objectForKey:key];
	id tempElement=[tempQueue objectAtIndex:0];
	[tempQueue removeObjectAtIndex:0];
		if ([tempQueue count]!=0) {
			[keysQueue setObject:tempQueue forKey:key];
			return tempElement;
		}else {
			[self delKey:key];
			return tempElement;
		}

}

- (id) popElementByKey:(NSString *)key: (NSString *)element{
	
	
	NSMutableArray *tempQueue=[keysQueue objectForKey:key];
	if (tempQueue!=NULL) {
		id element=[tempQueue objectAtIndex:0];
		[tempQueue removeObject:element];
		[keysQueue setObject:tempQueue forKey:key];
		return element;
	}
	else {
		NSLog(@"There is no element for %@ to return!",key);
		return nil;
	}

}
//----------------------------------------------------

-(NSMutableArray*) getQueue: (NSString *) key {
	//NSLog(@"here%@",[keysQueue objectForKey:key]);
	return [keysQueue objectForKey:key];
		
}
- (NSArray*)getKeys{

	return [keysQueue allKeys];
}
- (NSMutableDictionary *)allQueues{

	return keysQueue;
}
//----------------------------------------------------
- (NSInteger)size {
    return [keysQueue count];
}

- (BOOL)isEmpty {
    return [keysQueue count] == 0;
}

- (void)clear {
    [keysQueue removeAllObjects];
}


@end